@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2024 Martin Donath <martin.donath@squidfunk.com>
+# Copyright (c) 2016-2025 Martin Donath <martin.donath@squidfunk.com>
 
 # Permission is hereby granted, free of charge, to any person obtaining a copy
 # of this software and associated documentation files (the "Software"), to
@@ -26,7 +26,8 @@ import posixpath
 import yaml
 
 from babel.dates import format_date, format_datetime
-from datetime import datetime
+from copy import copy
+from datetime import datetime, timezone
 from jinja2 import pass_context
 from jinja2.runtime import Context
 from mkdocs.config.defaults import MkDocsConfig
@@ -34,19 +35,25 @@ from mkdocs.exceptions import PluginError
 from mkdocs.plugins import BasePlugin, event_priority
 from mkdocs.structure import StructureItem
 from mkdocs.structure.files import File, Files, InclusionLevel
-from mkdocs.structure.nav import Navigation, Section
+from mkdocs.structure.nav import Link, Navigation, Section
 from mkdocs.structure.pages import Page
+from mkdocs.structure.toc import AnchorLink, TableOfContents
 from mkdocs.utils import copy_file, get_relative_url
-from mkdocs.utils.templates import url_filter
 from paginate import Page as Pagination
 from shutil import rmtree
 from tempfile import mkdtemp
+from urllib.parse import urlparse
 from yaml import SafeLoader
 
-from .author import Authors
+from . import view_name
+from .author import Author, Authors
 from .config import BlogConfig
 from .readtime import readtime
-from .structure import Archive, Category, Excerpt, Post, View
+from .structure import (
+  Archive, Category, Profile,
+  Excerpt, Post, View,
+  Reference
+)
 
 # -----------------------------------------------------------------------------
 # Classes
@@ -84,12 +91,6 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         if self.config.authors:
             self.authors = self._resolve_authors(config)
 
-        # Initialize table of contents settings
-        if not isinstance(self.config.archive_toc, bool):
-            self.config.archive_toc = self.config.blog_toc
-        if not isinstance(self.config.categories_toc, bool):
-            self.config.categories_toc = self.config.blog_toc
-
         # By default, drafts are rendered when the documentation is served,
         # but not when it is built, for a better user experience
         if self.is_serve and self.config.draft_on_serve:
@@ -132,27 +133,40 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         self.blog = self._resolve(files, config)
         self.blog.posts = sorted(
             self._resolve_posts(files, config),
-            key = lambda post: post.config.date.created,
+            key = lambda post: (
+                post.config.pin,
+                post.config.date.created
+            ),
             reverse = True
         )
 
         # Generate views for archive
         if self.config.archive:
-            self.blog.views.extend(
-                self._generate_archive(config, files)
-            )
+            views = self._generate_archive(config, files)
+            self.blog.views.extend(views)
 
         # Generate views for categories
         if self.config.categories:
+            views = self._generate_categories(config, files)
+
+            # We always sort the list of categories by name first, so that any
+            # custom sorting function that returns the same value for two items
+            # returns them in a predictable and logical order, because sorting
+            # in Python is stable, i.e., order of equal items is preserved
             self.blog.views.extend(sorted(
-                self._generate_categories(config, files),
-                key = lambda view: view.name,
-                reverse = False
+                sorted(views, key = view_name),
+                key     = self.config.categories_sort_by,
+                reverse = self.config.categories_sort_reverse
             ))
 
+        # Generate views for profiles
+        if self.config.authors_profiles:
+            views = self._generate_profiles(config, files)
+            self.blog.views.extend(views)
+
         # Generate pages for views
-        if self.config.pagination:
-            for view in self._resolve_views(self.blog):
+        for view in self._resolve_views(self.blog):
+            if self._config_pagination(view):
                 for page in self._generate_pages(view, config, files):
                     view.pages.append(page)
 
@@ -207,9 +221,18 @@ class BlogPlugin(BasePlugin[BlogConfig]):
             if self.blog.file.inclusion.is_in_nav() and views:
                 self._attach_to(self.blog, Section(title, views), nav)
 
+        # Attach views for profiles
+        if self.config.authors_profiles:
+            title = self._translate(self.config.authors_profiles_name, config)
+            views = [_ for _ in self.blog.views if isinstance(_, Profile)]
+
+            # Attach and link views for categories, if any
+            if self.blog.file.inclusion.is_in_nav() and views:
+                self._attach_to(self.blog, Section(title, views), nav)
+
         # Attach pages for views
-        if self.config.pagination:
-            for view in self._resolve_views(self.blog):
+        for view in self._resolve_views(self.blog):
+            if self._config_pagination(view):
                 for at in range(1, len(view.pages)):
                     self._attach_at(view.parent, view, view.pages[at])
 
@@ -225,7 +248,7 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # Skip if page is not a post managed by this instance - this plugin has
         # support for multiple instances, which is why this check is necessary
         if page not in self.blog.posts:
-            if not self.config.pagination:
+            if not self._config_pagination(page):
                 return
 
             # We set the contents of the view to its title if pagination should
@@ -248,12 +271,12 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
         # Extract and assign authors to post, if enabled
         if self.config.authors:
-            for name in page.config.authors:
-                if name not in self.authors:
-                    raise PluginError(f"Couldn't find author '{name}'")
+            for id in page.config.authors:
+                if id not in self.authors:
+                    raise PluginError(f"Couldn't find author '{id}'")
 
                 # Append to list of authors
-                page.authors.append(self.authors[name])
+                page.authors.append(self.authors[id])
 
         # Extract settings for excerpts
         separator      = self.config.post_excerpt_separator
@@ -299,12 +322,20 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         if not self.config.enabled:
             return
 
+        # Transform links to point to posts and pages
+        for post in self.blog.posts:
+            self._generate_links(post, config, files)
+
         # Filter for formatting dates related to posts
         def date_filter(date: datetime):
             return self._format_date_for_post(date, config)
 
+        # Fetch URL template filter from environment - the filter might
+        # be overridden by other plugins, so we must retrieve and wrap it
+        url_filter = env.filters["url"]
+
         # Patch URL template filter to add support for paginated views, i.e.,
-        # that paginated views never link to themselves but to the main view
+        # that paginated views never link to themselves but to the main vie
         @pass_context
         def url_filter_with_pagination(context: Context, url: str | None):
             page = context["page"]
@@ -370,7 +401,7 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # and must be explicitly enabled by the author.
         if not isinstance(post.config.draft, bool):
             if self.config.draft_if_future_date:
-                return post.config.date.created > datetime.now()
+                return post.config.date.created > datetime.now(timezone.utc)
 
         # Post might be a draft
         return bool(post.config.draft)
@@ -524,14 +555,15 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
             # Create file for view, if it does not exist
             file = files.get_file_from_path(path)
-            if not file or self.temp_dir not in file.abs_src_path:
+            if not file:
                 file = self._path_to_file(path, config)
                 files.append(file)
 
-                # Create file in temporary directory and temporarily remove
-                # from navigation, as we'll add it at a specific location
+                # Create file in temporary directory
                 self._save_to_file(file.abs_src_path, f"# {name}")
-                file.inclusion = InclusionLevel.EXCLUDED
+
+            # Temporarily remove view from navigation
+            file.inclusion = InclusionLevel.EXCLUDED
 
             # Create and yield view
             if not isinstance(file.page, Archive):
@@ -560,14 +592,15 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
                 # Create file for view, if it does not exist
                 file = files.get_file_from_path(path)
-                if not file or self.temp_dir not in file.abs_src_path:
+                if not file:
                     file = self._path_to_file(path, config)
                     files.append(file)
 
-                    # Create file in temporary directory and temporarily remove
-                    # from navigation, as we'll add it at a specific location
+                    # Create file in temporary directory
                     self._save_to_file(file.abs_src_path, f"# {name}")
-                    file.inclusion = InclusionLevel.EXCLUDED
+
+                # Temporarily remove view from navigation
+                file.inclusion = InclusionLevel.EXCLUDED
 
                 # Create and yield view
                 if not isinstance(file.page, Category):
@@ -578,6 +611,37 @@ class BlogPlugin(BasePlugin[BlogConfig]):
                 file.page.posts.append(post)
                 post.categories.append(file.page)
 
+    # Generate views for profiles - analyze posts and generate the necessary
+    # views to provide a profile page for each author listing all posts
+    def _generate_profiles(self, config: MkDocsConfig, files: Files):
+        for post in self.blog.posts:
+            for id in post.config.authors:
+                author = self.authors[id]
+                path = self._format_path_for_profile(id, author)
+
+                # Create file for view, if it does not exist
+                file = files.get_file_from_path(path)
+                if not file:
+                    file = self._path_to_file(path, config)
+                    files.append(file)
+
+                    # Create file in temporary directory
+                    self._save_to_file(file.abs_src_path, f"# {author.name}")
+
+                # Temporarily remove view from navigation and assign profile
+                # URL to author, if not explicitly set
+                file.inclusion = InclusionLevel.EXCLUDED
+                if not author.url:
+                    author.url = file.url
+
+                # Create and yield view
+                if not isinstance(file.page, Profile):
+                    yield Profile(author.name, file, config)
+
+                # Assign post to profile
+                assert isinstance(file.page, Profile)
+                file.page.posts.append(post)
+
     # Generate pages for pagination - analyze view and generate the necessary
     # pages, creating a chain of views for simple rendering and replacement
     def _generate_pages(self, view: View, config: MkDocsConfig, files: Files):
@@ -585,20 +649,21 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
         # Compute pagination boundaries and create pages - pages are internally
         # handled as copies of a view, as they map to the same source location
-        step = self.config.pagination_per_page
+        step = self._config_pagination_per_page(view)
         for at in range(step, len(view.posts), step):
             path = self._format_path_for_pagination(view, 1 + at // step)
 
             # Create file for view, if it does not exist
             file = files.get_file_from_path(path)
-            if not file or self.temp_dir not in file.abs_src_path:
+            if not file:
                 file = self._path_to_file(path, config)
                 files.append(file)
 
-                # Copy file to temporary directory  and temporarily remove
-                # from navigation, as we'll add it at a specific location
+                # Copy file to temporary directory
                 copy_file(view.file.abs_src_path, file.abs_src_path)
-                file.inclusion = InclusionLevel.EXCLUDED
+
+            # Temporarily remove view from navigation
+            file.inclusion = InclusionLevel.EXCLUDED
 
             # Create and yield view
             if not isinstance(file.page, View):
@@ -609,6 +674,79 @@ class BlogPlugin(BasePlugin[BlogConfig]):
             file.page.pages = view.pages
             file.page.posts = view.posts
 
+    # Generate links from the given post to other posts, pages, and sections -
+    # this can only be done once all posts and pages have been parsed
+    def _generate_links(self, post: Post, config: MkDocsConfig, files: Files):
+        if not post.config.links:
+            return
+
+        # Resolve path relative to docs directory for error reporting
+        docs = os.path.relpath(config.docs_dir)
+        path = os.path.relpath(post.file.abs_src_path, docs)
+
+        # Find all links to pages and replace them with references - while all
+        # internal links are processed, external links remain as they are
+        for link in _find_links(post.config.links.items):
+            url = urlparse(link.url)
+            if url.scheme:
+                continue
+
+            # Resolve file for link, and throw if the file could not be found -
+            # authors can link to other pages, as well as to assets or files of
+            # any kind, but it is essential that the file that is linked to is
+            # found, so errors are actually catched and reported
+            file = files.get_file_from_path(url.path)
+            if not file:
+                log.warning(
+                    f"Error reading metadata of post '{path}' in '{docs}':\n"
+                    f"Couldn't find file for link '{url.path}'"
+                )
+                continue
+
+            # If the file linked to is not a page, but an asset or any other
+            # file, we resolve the destination URL and continue
+            if not isinstance(file.page, Page):
+                link.url = file.url
+                continue
+
+            # Cast link to reference
+            link.__class__ = Reference
+            assert isinstance(link, Reference)
+
+            # Assign page title, URL and metadata to link
+            link.title = link.title or file.page.title
+            link.url   = file.page.url
+            link.meta  = copy(file.page.meta)
+
+            # If the link has no fragment, we can continue - if it does, we
+            # need to find the matching anchor in the table of contents
+            if not url.fragment:
+                continue
+
+            # If we're running under dirty reload, MkDocs will reset all pages,
+            # so it's not possible to resolve anchor links. Thus, the only way
+            # to make this work is to skip the entire process of anchor link
+            # resolution in case of a dirty reload.
+            if self.is_dirty:
+                continue
+
+            # Resolve anchor for fragment, and throw if the anchor could not be
+            # found - authors can link to any anchor in the table of contents
+            anchor = _find_anchor(file.page.toc, url.fragment)
+            if not anchor:
+                log.warning(
+                    f"Error reading metadata of post '{path}' in '{docs}':\n"
+                    f"Couldn't find anchor '{url.fragment}' in '{url.path}'"
+                )
+
+                # Restore link to original state
+                link.url = url.geturl()
+                continue
+
+            # Append anchor to URL and set subtitle
+            link.url += f"#{anchor.id}"
+            link.meta["subtitle"] = anchor.title
+
     # -------------------------------------------------------------------------
 
     # Attach a list of pages to each other and to the given parent item without
@@ -661,11 +799,11 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         posts, pagination = view.posts, None
 
         # Create pagination, if enabled
-        if self.config.pagination:
+        if self._config_pagination(view):
             at = view.pages.index(view)
 
             # Compute pagination boundaries
-            step = self.config.pagination_per_page
+            step = self._config_pagination_per_page(view)
             p, q = at * step, at * step + step
 
             # Extract posts in pagination boundaries
@@ -685,18 +823,9 @@ class BlogPlugin(BasePlugin[BlogConfig]):
     def _render_post(self, excerpt: Excerpt, view: View):
         excerpt.render(view, self.config.post_excerpt_separator)
 
-        # Determine whether to add posts to the table of contents of the view -
-        # note that those settings can be changed individually for each type of
-        # view, which is why we need to check the type of view and the table of
-        # contents setting for that type of view
-        toc = self.config.blog_toc
-        if isinstance(view, Archive):
-            toc = self.config.archive_toc
-        if isinstance(view, Category):
-            toc = self.config.categories_toc
-
         # Attach top-level table of contents item to view if it should be added
         # and both, the view and excerpt contain table of contents items
+        toc = self._config_toc(view)
         if toc and excerpt.toc.items and view.toc.items:
             view.toc.items[0].children.append(excerpt.toc.items[0])
 
@@ -720,6 +849,48 @@ class BlogPlugin(BasePlugin[BlogConfig]):
 
     # -------------------------------------------------------------------------
 
+    # Retrieve configuration value or return default
+    def _config(self, key: str, default: any):
+        return default if self.config[key] is None else self.config[key]
+
+    # Retrieve configuration value for table of contents
+    def _config_toc(self, view: View):
+        default = self.config.blog_toc
+        if isinstance(view, Archive):
+            return self._config("archive_toc", default)
+        if isinstance(view, Category):
+            return self._config("categories_toc", default)
+        if isinstance(view, Profile):
+            return self._config("authors_profiles_toc", default)
+        else:
+            return default
+
+    # Retrieve configuration value for pagination
+    def _config_pagination(self, view: View):
+        default = self.config.pagination
+        if isinstance(view, Archive):
+            return self._config("archive_pagination", default)
+        if isinstance(view, Category):
+            return self._config("categories_pagination", default)
+        if isinstance(view, Profile):
+            return self._config("authors_profiles_pagination", default)
+        else:
+            return default
+
+    # Retrieve configuration value for pagination per page
+    def _config_pagination_per_page(self, view: View):
+        default = self.config.pagination_per_page
+        if isinstance(view, Archive):
+            return self._config("archive_pagination_per_page", default)
+        if isinstance(view, Category):
+            return self._config("categories_pagination_per_page", default)
+        if isinstance(view, Profile):
+            return self._config("authors_profiles_pagination_per_page", default)
+        else:
+            return default
+
+    # -------------------------------------------------------------------------
+
     # Format path for post
     def _format_path_for_post(self, post: Post, config: MkDocsConfig):
         categories = post.config.categories[:self.config.post_url_max_categories]
@@ -759,6 +930,17 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         path = posixpath.normpath(path.strip("/"))
         return posixpath.join(self.config.blog_dir, f"{path}.md")
 
+    # Format path for profile
+    def _format_path_for_profile(self, id: str, author: Author):
+        path = self.config.authors_profiles_url_format.format(
+            slug = author.slug or id,
+            name = author.name
+        )
+
+        # Normalize path and strip slashes at the beginning and end
+        path = posixpath.normpath(path.strip("/"))
+        return posixpath.join(self.config.blog_dir, f"{path}.md")
+
     # Format path for pagination
     def _format_path_for_pagination(self, view: View, page: int):
         path = self.config.pagination_url_format.format(
@@ -864,6 +1046,35 @@ class BlogPlugin(BasePlugin[BlogConfig]):
         # Translate placeholder
         return template.module.t(key)
 
+# -----------------------------------------------------------------------------
+# Helper functions
+# -----------------------------------------------------------------------------
+
+# Find all links in the given list of items
+def _find_links(items: list[StructureItem]):
+    for item in items:
+
+        # Resolve link
+        if isinstance(item, Link):
+            yield item
+
+        # Resolve sections recursively
+        if isinstance(item, Section):
+            for item in _find_links(item.children):
+                assert isinstance(item, Link)
+                yield item
+
+# Find anchor in table of contents for the given id
+def _find_anchor(toc: TableOfContents, id: str):
+    for anchor in toc:
+        if anchor.id == id:
+            return anchor
+
+        # Resolve anchors recursively
+        anchor = _find_anchor(anchor.children, id)
+        if isinstance(anchor, AnchorLink):
+            return anchor
+
 # -----------------------------------------------------------------------------
 # Data
 # -----------------------------------------------------------------------------
